//---------------------------------------------------------------------------
#include "stm32f4xx.h"
#include <stdio.h>
//---------------------------------------------------------------------------
void HSI_init(void);
void Pins_init(void);
void PC0_init(void);
void ADC_init(void);
int ADC_read(void);
float getTemperature(uint32_t adc_value);
void USART2_init(void); 
int USART2_write(int ch); 
int USART2_read(void);
void delayMs(int n);
//---------------------------------------------------------------------------
int main()
{
	int result;
	float tempr;
	  
	HSI_init();
	USART2_init();
	Pins_init();
	ADC_init();
	
	while(1)
	{
		result = ADC_read();
		tempr = getTemperature(result);
		printf("Temperature: %.2fc\r\n", tempr);
	}
}
//---------------------------------------------------------------------------
void HSI_init(void)  //Set Clock to 16Mhz
{
	// Set the HSION bit and wait for HSIRDY
	RCC->CR |= 1ul << 0;
	while(!(RCC->CR & 2ul)){};

  // System clock switch to HSI		
  RCC->CFGR &= ~(3ul << 0);   // HSI oscillator selected as system clock
	RCC->CFGR &= ~(7ul << 27);  // MCO2 Prescaler. NO DIVISION selected 
	RCC->CFGR &= ~(3ul << 30);  // System clock (SYSCLK) selected
}
//--------------------------------------------------------------------------
void Pins_init(void)
{
	//SYSCFGEN: System configuration controller clock enable 
	RCC->APB2ENR |= RCC_APB2ENR_SYSCFGEN; 
	
	PC0_init();			//Analogue Input
}
//--------------------------------------------------------------------------
void PC0_init(void)
{
	//Setup pin PC0 for analog input
	RCC->AHB1ENR |= 4; //Enable GPIOC clock
	GPIOC->MODER |= 3; //PC0 analog
}
//-----------------------------------------------------------------------------
void ADC_init(void)
{	
	//Setup ADC1
	RCC->APB2ENR |= RCC_APB2ENR_ADC1EN;   // Enable ADC1 clock

	// Select the ADC channel 16 (internal temperature sensor)
	ADC->CCR |= ADC_CCR_TSVREFE;          // Enable temperature sensor (Temperature Sensor and VREFINT Enable)
	ADC1->SQR3 = 10;                      // Channel 10 for external temperature sensor
	ADC1->SMPR1 |= 0;  										// Sampling time of 3 cycles

	// Enable ADC1
	ADC1->CR2 |= ADC_CR2_ADON;
}
//-----------------------------------------------------------------------------
int ADC_read(void)
{
	int result;
	
	ADC1->CR2 |= 0x40000000; //Start conversion
	while(!(ADC1->SR & 2));  //Wait for conversion to complete
	result = ADC1->DR;       //Read conversion
	
	return result;
}
//-----------------------------------------------------------------------------
float getTemperature(uint32_t adc_value)
{
	float tempr;
	
	tempr = (double)adc_value / 4095 * 330;
  
	return tempr;
}
//--------------------------------------------------------------------------
// USART2 Initialization for 9600 baud rate
void USART2_init(void) 
{
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN; // Enable GPIOA clock
	RCC->APB1ENR |= RCC_APB1ENR_USART2EN; // Enable USART2 clock
	
	// Configure PA2 (USART2_TX) and PA3 (USART2_RX)
	GPIOA->AFR[0] |= 0x7700; // Set alternate function for PA2 and PA3 to USART2
	GPIOA->MODER |= 0x00A0; // Set PA2 and PA3 to alternate function mode
	USART2->BRR = 0x0683; // 9600 baud rate @ 16 MHz
	USART2->CR1 = USART_CR1_TE | USART_CR1_RE | USART_CR1_RXNEIE; // Enable TX, RX and RX interrupt
	USART2->CR1 |= USART_CR1_UE; // Enable USART2
	
	NVIC_EnableIRQ(USART2_IRQn); // Enable USART2 interrupt in NVIC
}
//--------------------------------------------------------------------------
int USART2_write(int ch) 
{ 
	while(!(USART2->SR & 0x0080)); //wait until TX buffer empty 
	USART2->DR = (ch & 0xFF); 
	
	return ch; 
} 
//--------------------------------------------------------------------------
int USART2_read(void) 
{ 
	while(!(USART2->SR & 0x0020)); //wait until char arrives 
	
	return USART2->DR; 
}
//--------------------------------------------------------------------------
// Interface to the C standard I/O library. All I/O are directed to the console, which is USART2
struct __FILE { int handle; };
FILE __stdout = { 1 };
FILE __stdin = { 0 };
//--------------------------------------------------------------------------
// Called by C library console output
int fputc(int c, FILE *f) 
{
	while(!(USART2->SR & USART_SR_TXE));
	USART2->DR = c;
	
	return c;
}
//--------------------------------------------------------------------------
int fgetc(FILE *f) 
{
	while (!(USART2->SR & USART_SR_RXNE));
	
	return USART2->DR;
}
//--------------------------------------------------------------------------
void USART2_IRQHandler(void) 
{
	if(USART2->SR & USART_SR_RXNE) 
	{ 	
		// Check if data is received
		char c = getchar(); // Read received character
		
		if(c)
		{			
			printf("\r\nFeedback: '%c' \r\n",c);
		}
	}
}
//---------------------------------------------------------------------------
void delayMs(int n) 
{
	int i;
	for (; n > 0; n--)
	for (i = 0; i < 3195; i++);
}
//---------------------------------------------------------------------------
